function getParameterByName(name)
{
  name = name.replace(/[\[]/, "\\\[").replace(/[\]]/, "\\\]");
  var regexS = "[\\?&]" + name + "=([^&#]*)";
  var regex = new RegExp(regexS);
  var results = regex.exec(window.location.search);
  if(results == null)
    return "";
  else
    return decodeURIComponent(results[1].replace(/\+/g, " "));
}


Ext.define('DSS.addon.ConsumptionMessenger', {

    extend : 'DSS.addon.Framework',

    config : {
        appName : 'Messages',
        appId : 'hleedit',
        appVersion : '1.0.0.0',
        appIcon : 'default_icon.png'
    },

    getHelp : function() {
        return Ext.create('Ext.Component', {
            whiteSpace : 'normal',
            padding : '0 20 0 0', //top right bottom left
            loader : {
                url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
                autoLoad : true,
                disableCaching : false
            }
        });
    },

    getContent : function() {
        var me=this;
        
        me.recurrenceModes = [
                {'modeID':1, 'modeName': _('weekly')},
                {'modeID':2, 'modeName': _('daily')},
                {'modeID':3, 'modeName': _('hourly')},
                {'modeID':4, 'modeName': _('minutly')}];
        
        
        var oConfig={
                defaultSortCol:'name',
                disabledProperty:'disabled',
                storeConfig: {
                    storeId : 'store',
                    fields : [ {name:'disabled', sortType:'asBoolean'}, 
                               {name:'name', sortType:'asUCText'}, 
                               {name:'template', sortType:'asUCText'}, 
                               {name:'dsid', sortType:'asUCText'}, 
                               {name:'dsuid', sortType:'asUCText'}, 
                               {name:'devicename', sortType:'asUCText'},
                               {name:'recurrence', sortType:'asUCText'} ]
                },
                multiselect:true,
                columns: [{
                    header : _('Active'),
                    dataIndex : 'disabled',
                    resizable : false,
                    xtype: 'disabledcolumn', 
                    width : 50
                }, {
                    header : _('Name'),
                    dataIndex : 'name',
                    xtype: 'textcolumn', 
                    width : 200
                },{
                    header : _('template'),
                    dataIndex : 'template',
                    xtype: 'textcolumn', 
                    width : 200
                }, {
                    header : _('Device name'),
                    dataIndex : 'devicename',
                    renderer: function (val, metadata, record, rowIdx, colIdx, store, view) {  
                        var oDevice = dss.staticDataModel.getDevice(record.get('dsuid'));
                        var sError="";
                        if (oDevice.present==false) {
                        	sError="color:#aaaaaa;";
                        }
                        var sString = '<table><tr><td style="padding-right:5px;'+sError+'"><img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '"></td>';
                        sString += '<td style="'+sError+'">' + oDevice.name + _(" in ");
                        sString += dss.staticDataModel.getNameForZone(oDevice.ZoneID) + '</td></tr></table>'; 
                        
                        return sString;
                    },
                    flex : 3
                }, {
                    header: _('dsuid'),
                    dataIndex: 'dsuid',
                    xtype: 'devicedsidcolumn', 
                    width: 70,
                    customColumnRenderer: function (val, metadata, record, rowIdx, colIdx, store, view) {   
                    	 var oDevice = dss.staticDataModel.getDevice(record.get('dsuid'));
                         var sError="";
                         if (oDevice.present==false) {   
                          	return '<span style="color:#aaaaaa;">'+val.substr(val.length-8) + "</span>";
                         } else
                        	 return val.slice(val.length-10, val.length-2);
                    }
                }, {
                    header : _('recurrence'),
                    dataIndex : 'recurrence',
                    xtype: 'textcolumn', 
                    width : 200,
                    customColumnRenderer: function (val, metadata, record, rowIdx, colIdx, store, view) {   
                   	 switch (val) {
						case "1":
				   	 		return  _('weekly');
				   	 	break;
						case "2":
				   	 		return  _('daily');
				   	 	break;
						case "3":
				   	 		return  _('hourly');
				   	 	break;
						case "4":
				   	 		return  _('minutly');
				   	 	break;
                   	 }
                   }
                }],     
                buttonText: {
                    newButton: _('New message'),
                    editButton: _('Edit message'),
                    enableButton:_('Activate message'),
                    disableButton:_('Deactivate message'),
                    testButton: _('Test message'),
                    deleteButton: _('Delete message')
                },
                editorConfig:{
                    title : _('Messageproperties'),
                    staticControls : [{
                        type : 'text',
                        label : _('Name'),
                        id : 'name'
                    },{
                        type : 'text',
                        label : _('Template'),
                        id : 'template'
                    }, {
                        type : 'combobox',
                        label : _('recurrence'),
                        id : 'recurrence',
                        data : me.recurrenceModes,
                        storevalueid : 'modeID',
                        storedisplayid : 'modeName'
                    }],
                    tabControls :  [{
                        tabLabel : _('Device'),
                        tabDescription : _('Select the device for the message'),
                        content : [{
                            type : 'deviceSelection',
                            id : 'dsuid', 
                            filterFunc: function (device) {return ([5320,5340].indexOf(device.productID)!=-1);}, 
                            specialDeviceRenderer:   function(val, metadata, record, colIdx, store, view) {
                                var sReturn='<table style="height:16px;"><tr>';
                                sReturn+='<td><img src="' + record.get('icon') + '" style="margin-right: 5px;"></td>';
                                sReturn+='<td width="100%">' + record.get('name') + '</td><td>';
                                sReturn+=dss.staticDataModel.getProductNameForDevice(dss.staticDataModel.getDevice(record.get('dSUID')))+'</td>';                               
                                return sReturn +'</tr></table>';      
                            },
                            flex : true
                       }]
                    }]
                },
                listeners: {
                    'loadDataStore':function (grid){
                        me.loadParameter(grid);
                    },
                    'saveEntry':function (grid,values){    
                        
                        dss.raiseEvent('consumption-messenger', {
                            actions : 'save',
                            value : Ext.JSON.encode(values)
                        });
                        me.waitAndReload(grid);
                    },
                    'enableEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('dsuid');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('dsuid'));
                    	}
                        dss.raiseEvent('consumption-messenger', {
                            actions : 'enable',
                            id : Ext.JSON.encode(idVal)
                        });
                        me.waitAndReload(grid);
                     },
                    'disableEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=entry.get('dsuid');
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('dsuid'));
                    	}
                        dss.raiseEvent('consumption-messenger', {
                            actions : 'disable',
                            id : Ext.JSON.encode(idVal)
                        });
                        me.waitAndReload(grid);
                    },
                    'deleteEntry':function (grid,entry){
                    	var idVal;
                    	if (entry.get!=null) {
                    		idVal=[entry.get('dsuid')];
                    	} else {
                    		idVal=[];
                    		for (var iIndex=0;iIndex<entry.length;iIndex++) 
                    			if (entry[iIndex]!=null)
                    				idVal.push(entry[iIndex].get('dsuid'));
                    	}
                        dss.raiseEvent('consumption-messenger', {
                            actions : 'delete',
                            id : Ext.JSON.encode(idVal)
                        });
                        me.waitAndReload(grid);
                    },
                    'testEntry':function (grid,entry){
                        dss.raiseEvent('consumption-messenger', {
                            actions : 'test',
                            id : entry.get('dsuid')
                        });
                    }
                }
        };
        if (getParameterByName('mode')=='expert') {
        	oConfig.grouperField='grouping';
        	oConfig.editorConfig.tabControls[0].content[0].userStates=true;
        }        
        return Ext.create('DSS.gridPanel', oConfig);
    },
    loadParameter : function(grid) {
    	grid.setStatus(_('Loading ...'));
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/scripts/consumption-messenger/entries/*(*)/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);      
        
        oResultObject=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                           {name:'template',type:'simple'},
                                                           {name:'disabled',type:'condition/enabled'},
                                                           {name:'dsid', type:'simple'}, 
                                                           {name:'dsuid', type:'simple'},
                                                           {name:'recurrence', type:'simple'}]); 
        if (oResultObject!=null) {
        	for (var iIndex=0;iIndex<oResultObject.length;iIndex++){
        		oResultObject[iIndex].devicename=dss.staticDataModel.getDevice(oResultObject[iIndex].dsuid).name;
        	}
            grid.loadParameter(oResultObject);
        }  
        grid.setStatus(_('Ready'));
    },
    waitAndReload : function(grid) {
    	grid.setStatus(_('Saving ...'));
        var me = this;
        var iSubscriptID = Math.floor(Math.random() * 100000000000);
        dss.ajaxSyncRequest('/json/event/subscribe', {
            subscriptionID : iSubscriptID,
            name : 'consumption-messenenger.saved'
        });
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
            subscriptionID : iSubscriptID,
            timeout : 60 * 1000
        }, function() {
            me.loadParameter(grid);
            dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
                subscriptionID : iSubscriptID,
                name : 'consumption-messenenger.saved'
            });
        });
        
    }
});

Ext.onReady(function() {
    dss.buildUpLang(['locale/{languageSuffix}/consumption-messenger.po']);
    var EmailEdit = Ext.create('DSS.addon.ConsumptionMessenger');
    EmailEdit.initPage();
});
